#define _LCD_FIRST_ROW          0x80     //Move cursor to the 1st row
#define _LCD_SECOND_ROW         0xC0     //Move cursor to the 2nd row
#define _LCD_THIRD_ROW          0x94     //Move cursor to the 3rd row
#define _LCD_FOURTH_ROW         0xD4     //Move cursor to the 4th row
#define _LCD_CLEAR              0x01     //Clear display
#define _LCD_RETURN_HOME        0x02     //Return cursor to home position, returns a shifted display to
                                         //its original position. Display data RAM is unaffected.
#define _LCD_CURSOR_OFF         0x0C     //Turn off cursor
#define _LCD_UNDERLINE_ON       0x0E     //Underline cursor on
#define _LCD_BLINK_CURSOR_ON    0x0F     //Blink cursor on
#define _LCD_MOVE_CURSOR_LEFT   0x10     //Move cursor left without changing display data RAM
#define _LCD_MOVE_CURSOR_RIGHT  0x14     //Move cursor right without changing display data RAM
#define _LCD_TURN_ON            0x0C     //Turn Lcd display on
#define _LCD_TURN_OFF           0x08     //Turn Lcd display off
#define _LCD_SHIFT_LEFT         0x18     //Shift display left without changing display data RAM
#define _LCD_SHIFT_RIGHT        0x1E     //Shift display right without changing display data RAM

// LCD Definitions
#define LCD_ADDR 0x4E

// Lcd constants
const char txt1[] = "I have to connect 40x4 lcd to my project";
const char txt2[] = "The lcd have 2 Enable pins ,";
const char txt3[] = "EN1 for rows 1&2 and EN2 for rows 3&4";
const char txt4[] = "How can I connect it to my Board? ";

char txt[42];

const char character[] = {0,10,21,17,10,4,0,0};

// Module variables
char hours, minutes, seconds, day, week, month, year;    // Global date/time variables

void I2C_LCD_Send_Nibble_1(char rs, char out_char) {
    I2C1_Start();
    I2C1_Is_Idle();
    I2C1_Write(LCD_ADDR);
    I2C1_Is_Idle();
    I2C1_Write((out_char & 0xF0) | rs | 0x02 | 0x08 | 0x04);
    I2C1_Is_Idle();
    Delay_us(200);
    I2C1_Write((out_char & 0xF0) | rs | 0x00 | 0x08 | 0x04);
    I2C1_Is_Idle();
    Delay_us(200);
    I2C1_Write((out_char & 0xF0) | rs | 0x02 | 0x08 | 0x04);
    I2C1_Stop();
}

void I2C_LCD_Send_Nibble_2(char rs, char out_char) {
    I2C1_Start();
    I2C1_Is_Idle();
    I2C1_Write(LCD_ADDR);
    I2C1_Is_Idle();
    I2C1_Write((out_char & 0xF0) | rs | 0x04 | 0x08 | 0x02);
    I2C1_Is_Idle();
    Delay_us(200);
    I2C1_Write((out_char & 0xF0) | rs | 0x00 | 0x08 | 0x02);
    I2C1_Is_Idle();
    Delay_us(200);
    I2C1_Write((out_char & 0xF0) | rs | 0x04 | 0x08 | 0x02);
    I2C1_Stop();
}

void I2C_LCD_Send_Nibbles_1(char rs, char out_char) {

    char hi_n = 0, lo_n = 0;

    hi_n = out_char & 0xF0;
    lo_n = out_char << 4;

    I2C1_Start();
    I2C1_Is_Idle();
    I2C1_Write(LCD_ADDR);
    I2C1_Is_Idle();
    Delay_us(80);
    I2C1_Write(hi_n | rs | 0x00 | 0x08 | 0x04);
    I2C1_Is_Idle();
    Delay_us(20);
    I2C1_Write(hi_n | rs | 0x02 | 0x08 | 0x04);
    I2C1_Is_Idle();
    Delay_us(80);
    I2C1_Write(lo_n | rs | 0x00 | 0x08 | 0x04);
    I2C1_Is_Idle();
    Delay_us(80);
    I2C1_Write(lo_n | rs | 0x02 | 0x08 | 0x04);
    I2C1_Stop();
}

void I2C_LCD_Send_Nibbles_2(char rs, char out_char) {
    char hi_n = 0, lo_n = 0;

    hi_n = out_char & 0xF0;
    lo_n = out_char << 4;

    I2C1_Start();
    I2C1_Is_Idle();
    I2C1_Write(LCD_ADDR);
    I2C1_Is_Idle();
    Delay_us(80);
    I2C1_Write(hi_n | rs | 0x00 | 0x08 | 0x02);
    I2C1_Is_Idle();
    Delay_us(20);
    I2C1_Write(hi_n | rs | 0x04 | 0x08 | 0x02);
    I2C1_Is_Idle();
    Delay_us(80);
    I2C1_Write(lo_n | rs | 0x00 | 0x08 | 0x02);
    I2C1_Is_Idle();
    Delay_us(80);
    I2C1_Write(lo_n | rs | 0x04 | 0x08 | 0x02);
    I2C1_Stop();
}

void I2C_LCD_Init() {

    Delay_ms(200);

    I2C_LCD_Send_Nibble_1(0,0x30); Delay_ms(200);
    I2C_LCD_Send_Nibble_1(0,0x30); Delay_ms(80);
    I2C_LCD_Send_Nibble_1(0,0x30); Delay_ms(80);
    I2C_LCD_Send_Nibble_1(0,0x20); Delay_ms(12);
    I2C_LCD_Send_Nibbles_1(0,0x01); Delay_ms(12);
    I2C_LCD_Send_Nibbles_1(0,0x28); Delay_ms(4);Delay_ms(12);
    I2C_LCD_Send_Nibbles_1(0,0x0C); Delay_ms(4);Delay_ms(12);
    I2C_LCD_Send_Nibbles_1(0,0X06); Delay_ms(4);Delay_ms(12);

    I2C_LCD_Send_Nibble_2(0,0x30); Delay_ms(200);
    I2C_LCD_Send_Nibble_2(0,0x30); Delay_ms(80);
    I2C_LCD_Send_Nibble_2(0,0x30); Delay_ms(80);
    I2C_LCD_Send_Nibble_2(0,0x20); Delay_ms(12);
    I2C_LCD_Send_Nibbles_2(0,0x01); Delay_ms(12);
    I2C_LCD_Send_Nibbles_2(0,0x28); Delay_ms(4);Delay_ms(12);
    I2C_LCD_Send_Nibbles_2(0,0x0C); Delay_ms(4);Delay_ms(12);
    I2C_LCD_Send_Nibbles_2(0,0X06); Delay_ms(4);Delay_ms(12);
}

void I2C_LCD_XY_OUT_1(char row, char column) {
    if(row == 1)I2C_LCD_Send_Nibbles_1(0,128 + column);
    if(row == 2)I2C_LCD_Send_Nibbles_1(0,192 + column);
    if(row == 3)I2C_LCD_Send_Nibbles_1(0,148 + column);
    if(row == 4)I2C_LCD_Send_Nibbles_1(0,212 + column);
    Delay_us(50);
}

void I2C_LCD_XY_OUT_2(char row, char column) {
    if(row == 1)I2C_LCD_Send_Nibbles_2(0,128 + column);
    if(row == 2)I2C_LCD_Send_Nibbles_2(0,192 + column);
    if(row == 3)I2C_LCD_Send_Nibbles_2(0,148 + column);
    if(row == 4)I2C_LCD_Send_Nibbles_2(0,212 + column);
    Delay_us(50);
}

void I2C_LCD_Cmd(char out_char) {
    I2C_LCD_Send_Nibbles_1(0,out_char);
    I2C_LCD_Send_Nibbles_2(0,out_char);
}

void I2C_LCD_Chr(char row, char column, char out_char) {
    if((row == 1) || (row == 2)) {
       if ((row == 1) && (column > 20)) {row = 3; column = column - 20;}
       if ((row == 2) && (column > 20)) {row = 4; column = column - 20;}
       I2C_LCD_XY_OUT_1(row,column-1);
       I2C_LCD_Send_Nibbles_1(1,out_char);
       Delay_us(40);
    }
    else {
       if (row == 3) row = 1;
       if (row == 4) row = 2;
       if ((row == 1) && (column > 20)) {row = 3; column = column - 20;}
       if ((row == 2) && (column > 20)) {row = 4; column = column - 20;}
       I2C_LCD_XY_OUT_2(row,column-1);
       I2C_LCD_Send_Nibbles_2(1,out_char);
       Delay_us(40);
    }
}

void I2C_LCD_Chr_Cp(char out_char) {
    I2C_LCD_Send_Nibbles_1(1,out_char);
    I2C_LCD_Send_Nibbles_2(1,out_char);
    Delay_us(40);
}


void I2C_LCD_Out(char row, char col, char *text) {
    while(*text)
         I2C_LCD_Chr(row, col++, *text++);
}

void I2C_LCD_Out_Cp(char *text) {
    while(*text)
         I2C_LCD_Chr_Cp(*text++);
}

void CustomChar(char pos_row, char pos_char) {
    char i;

    I2C_LCD_Cmd(64);
    for (i = 0; i<=7; i++) I2C_LCD_Chr_CP(character[i]);
    I2C_LCD_Cmd(_LCD_RETURN_HOME);
    I2C_LCD_Chr(pos_row, pos_char, 0);
    Delay_ms(2);
}

// copy const to ram string
char *CopyConst2Ram(char *dest, const char *src) {
    char *d;

    d = dest;
    for(;*dest++ = *src++;);
    return d;
}

void main() {

    TRISB = 0x0000;

    I2C1_Init(100000);

    I2C_LCD_Init();
    I2C_LCD_Cmd(_LCD_CURSOR_OFF);
    I2C_LCD_Cmd(_LCD_CLEAR);

    I2C_LCD_Out(1,1,CopyConst2Ram(txt, txt1));
    I2C_LCD_Out(2,1,CopyConst2Ram(txt, txt2));
    I2C_LCD_Out(3,1,CopyConst2Ram(txt, txt3));
    I2C_LCD_Out(4,1,CopyConst2Ram(txt, txt4));

    while(1) {

    }
}